<?php

namespace HCM\AVL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use HCM\Util\DateUtil;
use HCM\AVL\Services\QuestionarioService;
use Doctrine\ORM\EntityManager;

class QuestionarioController extends Crud {

    protected $entityManager;
    protected $questionarioService;

    public function __construct (Manager $dataSourceManager, EntityManager $entityManager, QuestionarioService $questionarioService){
        $this->entityManager = $entityManager;
        $this->questionarioService = $questionarioService;
    	parent::__construct($dataSourceManager);
    }

    public function getQuestionario(DTO\Request\Filter $request, DTO\Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
		$data = array("NRORG" => $filter[0]['value'], "NRAVALIACAO" => $filter[1]['value'], "NRPARCEXECAVAL" => $filter[2]['value'], "NRPARCAVALIADO" => $filter[3]['value']);
		$obj   = $this->constroiObjeto($data);
		$response->addDataSet(new DataSet("/questionario/get",array($obj)));
	}

	public function save(DTO\Request\DataSet $request, DTO\Response $response) {
		set_time_limit(0);
		$dataset         = $request->getDataSet()->getRows()[0];
		$nrorg           = $dataset["NRORG"];
		$cdoperador      = $dataset["CDOPERADOR"];
		$nravaliacao     = $dataset["NRAVALIACAO"];
		$nrcombquestaval = $dataset["NRCOMBQUESTAVAL"];
		$nrparcavaliado  = $dataset["NRPARCAVALIADO"];
		$nrparcexecaval  = $dataset["NRPARCEXECAVAL"];
		$dtexecucaoaval  = $dataset["DTEXECUCAOAVAL"];
		$dataAtual       = DateUtil::getDataAtual();
        $dataAtual       = $dataAtual->format("d/m/Y H:i:s");


		foreach ($dataset["answers"] as $questao => $resposta) {
			$nrquestao = str_replace("QUESTAO_", "", $questao);
			if(!empty($resposta)){
				$resposta = $resposta["value"];
				// getExecaval
				$nrexecavaliacao = $this->questionarioService->getExecaval($nrcombquestaval, $nrparcavaliado, $nravaliacao, $nrquestao, $nrparcexecaval, $nrorg);
	  			$nrexecavaliacao = $nrexecavaliacao["NREXECAVALIACAO"];
	  			// fim getExecaval

				// getQuestao
				$dsresposta = $this->questionarioService->getResposta($resposta, $nrorg);
	  			$dsresposta = $dsresposta["DSRESPOSTA"];
				// fim de getQuestao

				// getRespAvaliacao
				$result = $this->questionarioService->getRespAvaliacao($nrexecavaliacao, $nrorg);
				// fim getRespAvaliacao
	        	if ($result) {
	        		$this->questionarioService->updateRespAvaliacao($resposta, $dsresposta, $nrorg, $result["NRRESPAVALIACAO"]);
	        	} else {
	                $nrrespavaliacao = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'RESPAVALIACAO', 1, 22, false);
	                $this->questionarioService->insertRespavaliacao($nrrespavaliacao, $nrorg, $nrexecavaliacao, $resposta, $dsresposta, $dataAtual, $cdoperador);
	        	}
	        }
		}
		// update PARTICIPAVAL
    	$this->questionarioService->updateParticipaval($nravaliacao, $nrparcexecaval, $nrparcavaliado, $nrcombquestaval, $nrorg);
		// FIM update PARTICIPAVAL

		// update EXECAVALIACAO
    	$this->questionarioService->updateExecavaliacao($dataAtual, $nravaliacao, $nrparcexecaval, $nrparcavaliado, $nrcombquestaval, $nrorg);
		// frim update EXECAVALIACAO

		if (isset($dataset["concluir"])) {
			$this->enviar($nravaliacao, $nrparcexecaval, $nrparcavaliado, $nrcombquestaval, $nrorg);
		}
	}

	private function enviar($nravaliacao, $nrparcexecaval, $nrparcavaliado, $nrcombquestaval, $nrorg) {
		$dataAtual       = DateUtil::getDataAtual();
        $dataAtual       = $dataAtual->format("d/m/Y H:i:s");
		$this->questionarioService->enviar($nravaliacao, $nrparcexecaval, $nrparcavaliado, $nrcombquestaval, $nrorg, $dataAtual);
	}

	public function constroiObjeto($dados = array()) {
		$obj = array();
		$cabecalhoQuestionario = $this->getCabecalhoQuestionario($dados["NRAVALIACAO"], $dados["NRORG"]);
		$obj["name"]        = $cabecalhoQuestionario["NMAVALIACAO"];
		$obj["description"] = $cabecalhoQuestionario["NMAVALIACAO"];

		$questoes           = $this->getQuestoes (
			$dados["NRAVALIACAO"], 
			$dados["NRORG"],
			$dados["NRPARCEXECAVAL"], 
			$dados["NRPARCAVALIADO"]
		);

		if ($questoes) {
			$obj["questions"] = array();
			foreach ($questoes as $questao) {
				$indice = count($obj["questions"]);
				$obj["questions"][$indice]["type"]                    = "unique";
				$obj["questions"][$indice]["group"]                   = $questao["NMAGRUPQUESTAVA"];
				$obj["questions"][$indice]["description"]             = $questao["TXQUESTAO"];
				$obj["questions"][$indice]["name"]                    = $questao["QUESTION_NAME"];
				$obj["questions"][$indice]["required"]                = true;
				$obj["questions"][$indice]["seqno"]                   = $questao["NRORDEQUESTAVAL"];
				$alternativas = $this->getAlternativasQuestoes($questao["NRQUESTAO"], $dados["NRORG"]);

				if ($alternativas) {
					$obj["questions"][$indice]["options"] = array();
					foreach ($alternativas as $alternativa) {
						$indiceOpt = count($obj["questions"][$indice]["options"]);
						$obj["questions"][$indice]["options"][$indiceOpt]["description"] = $alternativa["DSRESPOSTA"];
						$obj["questions"][$indice]["options"][$indiceOpt]["value"]       = $alternativa["NRRESPOSTA"];
						$obj["questions"][$indice]["options"][$indiceOpt]["seqno"]       = $alternativa["NRORDENARESPOST"];
					}
				}
			}
		}
		
		$dataset = $this->questionarioService->constroiObjeto($dados);			    
		if ($dataset) {
			$respostas = array();
			foreach ($dataset as $row) {
				$respostas[$row["QUESTAO"]] = array("value" => $row["RESPOSTA"]);
			}

			$obj["answer"] = $respostas;
		}

		return $obj;
	}

	private function getCabecalhoQuestionario($nravaliacao, $nrorg) {
		return $this->questionarioService->getCabecalhoQuestionario($nravaliacao, $nrorg);
	}

	public function getExecavaliacao($nravaliacao, $nrorg, $nrparnegocio) {
		return $this->questionarioService->getExecavaliacao($nravaliacao, $nrorg, $nrparnegocio);
	}

	private function getQuestoes($nravaliacao, $nrorg, $nravaliador, $nravaliado) {
		return $this->questionarioService->getQuestoes($nravaliacao, $nrorg, $nravaliador, $nravaliado);
	}

	public function getAlternativasQuestoes($nrquestao, $nrorg) {
		return $this->questionarioService->getAlternativasQuestoes($nrquestao, $nrorg);
	}

	public function insertExecavaliacao($dataSet) {
		$this->questionarioService->insertExecavaliacao($dataSet);
	}

	public function insertParticipaval($dataSet){
		$this->questionarioService->insertParticipaval($dataSet);
	}

	public function verificaExistenciaParticipaval($dataSet){
		return $this->questionarioService->verificaExistenciaParticipaval($dataSet);
	}

}