<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\FPA\Calculos\Ferias\FeriasColetiva;
use Doctrine\ORM\EntityManager;

class FeriasColetivaController extends Crud {

    protected $dataSourceGetFormFerias = 'fpa_formferiascolet_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC', 'P_CDOPERADOR', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $HCMParameters, EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->HCMParameters = $HCMParameters;
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }

    public function getFormferiascolet (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetFormFerias);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }    
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }  
            
            if($row['DSVINCNAOCONTEMPL']){
                $row['DSVINCNAOCONTEMPL'] = explode(",", $row['DSVINCNAOCONTEMPL']);
                foreach($row['DSVINCNAOCONTEMPL'] as $chave => $value)
                    $row['DSVINCNAOCONTEMPL'][$chave] = intval($value);
                
                $row['NMVINCNAOCONTEMPL'] = explode(",", $row['NMVINCNAOCONTEMPL']);
                
            }
            $diaFerias = DateUtil::getDataDeString($row['DTINIGOZOFIM'])->diff(DateUtil::getDataDeString($row['DTINIGOZOINI']));
            $row['DIASFERIAS'] = $diaFerias->format('%d') + 1;
        }
        $response->addDataSet(new DataSet($this->dataSourceGetFormFerias, $rows));
    }

    public function setFormFeriasColet (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_formferiascolet";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFORMFERIAS', $row)) {
                    if ($row['NRFORMFERIAS'] === '') {
                       $row['NRFORMFERIAS'] = \HCM\Util\NovoCodigo::geraCodigo($this->HCMParameters->getNrorgTrabalho(), 'FPA_FORMFERIASCOLET', 1, 12, true); 
                    }
                } else {
                    $row['NRFORMFERIAS'] = \HCM\Util\NovoCodigo::geraCodigo($this->HCMParameters->getNrorgTrabalho(), 'FPA_FORMFERIASCOLET', 1, 12, true); 
                }
                
                $row['CDOPERADOR'] = $row['CDOPERULTATU'];
                $row['IDRETROCESSO'] = 'N';
                $row['IDULTPGTOMES'] = 'N';
                $row['DTINIPERAQUISTIVO'] = null;
                if(!isset($row['NRSITUFUNCM'])){
                    $row['NRSITUFUNCM'] = null;
                };
            }
            if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
 
            if(!array_key_exists('DSFIM', $row)){
                $row['DSFIM'] = null;
            }
            if($row['DSORDEM'] === "LISTA_VINCULO"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
            }
            if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
                if(is_array($row['DSFIM'])) {
                    if(count($row['DSFIM']) > 0){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    } else {
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
            }
            if(array_key_exists('DSVINCNAOCONTEMPL', $row) && is_array($row['DSVINCNAOCONTEMPL']) && count($row['DSVINCNAOCONTEMPL']) > 0){
                $row['DSVINCNAOCONTEMPL'] = implode(',',$row['DSVINCNAOCONTEMPL']);
            }else{
                $row['DSVINCNAOCONTEMPL'] = null;
            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteFormFeriasColet (DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "fpa_formferiascolet";
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function preparaColetiva (DTO\Request\DataSet $request, DTO\Response $response) {
        // Tempo de execucao deve ser ilimitado.
        set_time_limit(0);
        try {
            $dataSet = $request->getDataSet()->getRows()[0];
            
            $feriasColetiva = new FeriasColetiva($this->entityManager, $this->HCMParameters, $dataSet);
            $feriasColetiva->preparaColetiva();
            
            $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('dsorigmsg' => 'FERIASCOLETIVA', 'nrorg' => $this->HCMParameters->getNrorgTrabalho()));
            
            $row = array();
            foreach ($message as $mensagem) {
                $row[] = array("NRVINCULOM" => $mensagem->getNrvinculom(), "NRMSGCALC" => $mensagem->getNrmsgcalc(), "DSDESCRICAO" => $mensagem->getDescricao(), "IDTIPOMSG" => $mensagem->getIdtipomsg());
            
                $this->entityManager->remove($mensagem);
                $this->entityManager->flush();
            }
            if($row){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $row));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function retrocedeColetiva (DTO\Request\DataSet $request, DTO\Response $response) {
        // Tempo de execucao deve ser ilimitado.
        set_time_limit(0);

        try {
            
            $dataSet = $request->getDataSet()->getRows()[0];
            
            $feriasColetiva = new FeriasColetiva($this->entityManager, $this->HCMParameters, $dataSet);
            $cancelaColetiva = $feriasColetiva->cancelaColetiva();
            
            if(!empty($cancelaColetiva)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($cancelaColetiva['MENSAGEM'], DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vinculosLiquido', array($cancelaColetiva['VINCULOS'])));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}