<?php

namespace HCM\FPA\Services;


use Doctrine\ORM\EntityManager;
use HCM\FPA\Services\LoteLancEventoService;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Repository\RateioLancamentoRepository;

class RateioLancamentoService {
    
    protected $entityManager;
    
    public function __construct(EntityManager $entityManager, LoteLancEventoService $loteLancEventoService,  HCMParameters $hcmParameters, RateioLancamentoRepository $rateioLancamentoRepository) {
        $this->entityManager = $entityManager;
        $this->loteLancEventoService  = $loteLancEventoService;
        $this->hcmParameters = $hcmParameters;
        $this->rateioLancamentoRepository = $rateioLancamentoRepository;
    }

    public function exportaLote($itens, $nrorg, $dtmescompetenc, $nreventom)
    {   
        foreach($itens as $item) {
            $itensLote[] = array(
                'NRVINCULOM' => $item['NRVINCULOM'],
                'NMVINCULOM' => '',
                'NREVENTOM' => $nreventom,
                'NMEVENTOH' => '',
                'VRREFFOLHALOTE' => null,
                'VREVENTOLOTE' => $item['VRVALOR']
            );
        }

        $lote = $this->loteLancEventoService->geraLoteLancEvento(
            $nrorg, $dtmescompetenc, 'FPA', 'EXPORTAÇÃO DO RATEIO DE LANÇAMENTO', 'PENDENTE', '', 1,  $itensLote
        );

    }

    public function validaQntdVinculosParaRateio($qntdVinculosEstrutGerenCargo, $estrutgeren, $ocupacao)
    {
        foreach ($qntdVinculosEstrutGerenCargo as $item) {
            if ($estrutgeren == $item['NRESTRUTGEREN'] && $ocupacao == $item['NRCARGO']) {
                $qntdVinculosRateio = $item['QNTDVINCULOS']; 
                return $qntdVinculosRateio;
            }
        }
        
    }

    public function calculaRateio($vrFinalLancamento, $porcentagemEstrutGeren, $porcentagemOcupacao, $qntdVinculosRateio)
    {
        try {
            $resultado = ($vrFinalLancamento * ($porcentagemEstrutGeren/100)) * ($porcentagemOcupacao/100);
            $resultado = $resultado/$qntdVinculosRateio;
            return $resultado;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }


    public function buscaLancamentosFiltro($row)
    {
        try {
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $lancamentosRateio = $this->entityManager->getRepository(Repositories::FPA_RATEIOLANC)->findBy(array('nrrateio' => $row['NRRATEIO'], 'nrorg' => $row['NRORG']));
            $lancamentosRateioFiltro = array();
            foreach($lancamentosRateio as $lancamento) {
                $dtLancamento  = (DateUtil::getDataDeString($lancamento->getDtlancamento(),DateUtil::FORMATO_BRASILEIRO,true))->format('d/m/Y');

                if((strlen($row['DTINICIO']) > 1) && (strlen($row['DTFIM']) > 1)) {
                    if ($dtLancamento >= $row['DTINICIO'] && $dtLancamento <= $row['DTFIM']) {
                        array_push($lancamentosRateioFiltro, $lancamento);
                    }
                }

                if((strlen($row['DTINICIO']) > 1) && (strlen($row['DTFIM']) < 1)) {
                    if ($dtLancamento >= $row['DTINICIO'] && $dtLancamento <= $this->hcmParameters->getUltimoDiaMes()) {
                        array_push($lancamentosRateioFiltro, $lancamento);
                    }
                }
            }

            return $lancamentosRateioFiltro;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    public function buscaLancamentosAtualCompetencia($row)
    {   
        try {
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $lancamentosRateio = $this->entityManager->getRepository(Repositories::FPA_RATEIOLANC)->findBy(array('nrrateio' => $row['NRRATEIO'], 'nrorg' => $row['NRORG']));
            $lancamentosRateioCompAtual = array();
            foreach($lancamentosRateio as $lancamento) {
                $dtLancamento  = (DateUtil::getDataDeString($lancamento->getDtlancamento(),DateUtil::FORMATO_BRASILEIRO,true))->format('d/m/Y');
                if (substr($dtLancamento, 3,9) == substr($dtmescompetenc, 3,9) ) {
                    array_push($lancamentosRateioCompAtual, $lancamento);
                }
            }

            return $lancamentosRateioCompAtual;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }        
    }

    public function calculaPorcentagem($nrorg, $nrrateio, $tabela)
    {
       if($tabela == 'FPA_RATEIOESTRUT') {
           $porcentagens = $this->rateioLancamentoRepository->selectPorcentagemEstrutura($nrorg, $nrrateio);
           foreach($porcentagens as $porcentagem) {
            $this->rateioLancamentoRepository->setPorcentagemEstrutura($nrorg, $nrrateio, $porcentagem['NRRATEIOESTRUT'],$porcentagem['PERCENT']);
           }
       } 

       if($tabela == 'FPA_RATEIOOCUP') {
           $porcentagens = $this->rateioLancamentoRepository->selectPorcentagemOcupacao($nrorg, $nrrateio);
           foreach($porcentagens as $porcentagem) {
               $this->rateioLancamentoRepository->setPorcentagemOcupacao($nrorg, $nrrateio, $porcentagem['NRRATEIOOCUP'],$porcentagem['PERCENT']);
           }
       }    
   }

   public function existeItensRateio($nrorg, $nrrateio)
    {
        $existeItemEstrutura =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOESTRUT');
        $existeItemLancamento =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOLANC');
        $existeItemOcupacao = $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOOCUP' );
        if($existeItemEstrutura || $existeItemLancamento || $existeItemOcupacao) {
            return true;
        }

        return false;
    }
}