<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 08/08/2016
 * Time: 14:17
 */

namespace HCM\VAB\Controllers;

use HCM\Util\Messages;
use HCM\VAB\Services\IntegracaoValeTransporteService;
use HCM\VAB\IntegracaoVT\Controlador;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class IntegracaoValeTransporteController extends Crud{

    protected $dataSourceName = 'vab_integtransph';
    protected $dataSourceName_get = 'getIntegtransp';
    protected $dts_getTipointegtransp = 'tipointegtransp_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, IntegracaoValeTransporteService $integracaoValeTransporteService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->integracaoValeTransporteService = $integracaoValeTransporteService;
        parent::__construct($dataSourceManager);
    }

    public function getIntegtransp(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function deleteIntegtransp(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    public function getTipointegtransp(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dts_getTipointegtransp);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function importaIntegtransp(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nmintegracao = $row['NMINTEGRACAO'];
                $message = array();
                
                Controlador::resetQtdeRegistrosImportados();
                
                foreach ($row['DSPATH'] as $arquivo) {
                    $row['DSPATH'] = [$arquivo];
                    $message = $this->integracaoValeTransporteService->importaIntegracao($nrorg, $dtmescompetenc,$row, $message);
                }
                
                array_unshift($message, ['MENSAGEM' => 'Total de '.Controlador::getQtdeRegistrosImportados().' registro(s) importado(s).']); 
                
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getMessage', [$message]));
                $response->addNotification(new DTO\Response\Notification(Messages::HCM00001_GER_IMPORTACAO_SUCESS));
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
} 